@@ -487,7 +487,9 @@ void VG_Mixture_Model::random_soft_init()
 
 
 bool VG_Mixture_Model::check_aitkens(void) {
+
         int last_index = logliks.size();
+
         double l_p1 =  logliks[last_index-1];
         double l_t =  logliks[last_index-2];
 
@@ -503,8 +505,10 @@ bool VG_Mixture_Model::check_aitkens(void) {
         double l_m1 = logliks[last_index-3];
         double a_t = (l_p1 - l_t)/(l_t - l_m1);
         double l_Inf = l_t + (l_p1 - l_t)/(1.0-a_t);
-        double val = std::abs((l_Inf - l_t));
-        return (bool)(val < tol_l);
+        double val = l_Inf - logliks[last_index];
+        bool aitkens_check = (0.0 <= val ) & (val < tol_l);
+        return aitkens_check;
+
 }
 
 
@@ -611,8 +615,9 @@ bool VG_Mixture_Model::track_lg(bool check)
     double l_m1 = logliks[last_index-3];
     double a_t = (l_p1 - l_t)/(l_t - l_m1);
     double l_Inf = l_t + (l_p1 - l_t)/(1.0-a_t);
-    double val = std::abs((l_Inf - l_t));
-    return (bool)(val < tol_l); 
+    double val = l_Inf - l_t;
+    bool aitkens_check = (0.0 <= val ) & (val < tol_l);
+    return aitkens_check;
   }
 }
 
